# !/bin/python
import time
import requests
import xml.etree.ElementTree as ET
import socket
from datetime import datetime
import urllib
from io import StringIO

from board import SCL, SDA
import busio
from PIL import Image, ImageDraw, ImageFont
import adafruit_ssd1306

#Modify the url to your JRiver instance
sURL = 'http://localhost:52199/MCWS/v1/Playback/Info?Zone=-1'
#If you set the authentication in th emedia network, add the details below.
#If not modify function geturl(url), for the proper call
sUsername = 'username'
sPassword = 'password'
debug = 0
#Set the demensions of your OLED 
WIDTH = 128
HEIGHT = 64

# Create the I2C interface.
i2c = busio.I2C(SCL, SDA)
 
# Create the SSD1306 OLED class.
# The first two parameters are the pixel width and pixel height.  Change these
# to the right size for your display!
disp = adafruit_ssd1306.SSD1306_I2C(WIDTH, HEIGHT, i2c)

# Init code for scrolling when needed
text_x = disp.width 
text_y = 2
t_start = time.time()

def readattrval(roota,branch):
	for child in roota.findall(branch):
		if debug == 1:
			print( child.attrib, child.text)
		return child.text

def geturl(url):
	#If you need credetials to access the media network use this line
	#r = requests.get(url, auth=(sUsername, sPassword))
	#If you don't need credentials, use this line
	r = requests.get(url)
	root = ET.fromstring(r.content)
	global PositionDisplay
	PositionDisplay = readattrval(root,"./Item[@Name='PositionDisplay']")
	global Bitrate 
	Bitrate = readattrval(root,"./Item[@Name='Bitrate']")
	global Bitdepth 
	Bitdepth = readattrval(root,"./Item[@Name='Bitdepth']")
	global SampleRate
	SampleRate = readattrval(root,"./Item[@Name='SampleRate']")
	global Artist
	Artist = readattrval(root,"./Item[@Name='Artist']")
	global Album
	Album = readattrval(root,"./Item[@Name='Album']")
	global Name
	Name = readattrval(root,"./Item[@Name='Name']")
	global Status
	Status = readattrval(root,"./Item[@Name='Status']")
	global ImageURL
	ImageURL = readattrval(root,"./Item[@Name='ImageURL']")
	global PositionMS
	PositionMS = readattrval(root,"./Item[@Name='PositionMS']")
	global DurationMS
	DurationMS = readattrval(root,"./Item[@Name='DurationMS']")
#	ZoneID,State,FileKey,NextFileKey
#	ElapsedTimeDisplay,RemainingTimeDisplay,TotalTimeDisplay,PlayingNowPosition
#	PlayingNowTracks,PlayingNowPositionDisplay,PlayingNowChangeCounter
#	Channels,Chapter,Volume,VolumeDisplay

def placeimage(cX,cY,imgSizeX,imgSizeY,imagePath):
	image = Image.new('1', (10,10))
	image = Image.open(imagePath).convert('1')
	image = image.resize((imgSizeX,imgSizeY), Image.ANTIALIAS)
	imageSize = image.size
	img.paste(image, (cX, cY, cX+imageSize[0], cY+imageSize[1]))
	
def dispimage(sleeptime):
	disp.image(img)
	disp.show()
	time.sleep(sleeptime)
	
def scroll(field,offset,x,MESSAGE):
	try:
		if len(field) >= 19:
			draw.text((int(text_x - x) , text_y + offset ), field+" ", font=font, fill=255)
		elif MESSAGE == "Starting up":
			draw.text((0 , text_y), MESSAGE + " ", font=font, fill=255)
		else:
			draw.text((0 , text_y + offset ), field +" ", font=font, fill=255)
	except:
		draw.text((0 , text_y + offset ), " ", font=font, fill=255)
#Added to avoid null exception on boot, when JRiver is not started yet	
try:
	geturl(sURL)
except:
	Artist = ""
	Name = ""
	Album = ""
	Status =""
	MESSAGE = ""

MESSAGE = max(Artist,Name,Album)

#Here you can rotate your display when needed, 0=0deg, 1=90deg, 2=180deg, 3=270deg
disp.rotation=0

# Clear display.
disp.fill(0)
disp.show()

img = Image.new('1', (WIDTH,HEIGHT))
draw = ImageDraw.Draw(img)

# Font declaration. If you change this, you need to modify all code and indexes!!!
font = ImageFont.truetype("/usr/share/fonts/truetype/dejavu/DejaVuSans-Bold.ttf", 11)
fontA = ImageFont.truetype("/usr/share/fonts/truetype/dejavu/DejaVuSans-Bold.ttf", 10)
fontB = ImageFont.truetype('/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf', 8)
fontC = ImageFont.truetype('/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf', 9)
size_x, size_y = draw.textsize(MESSAGE, font)

while True:
	try:
		geturl(sURL)
	except:
		a=1
	if Status == "Playing" or Status == 'Paused' or Status == 'Waiting':
		#If started via cron @reboot or as entry in /etc/rc.locale, not all fields are filled as JRiver
		# isn't running, so we need to deal with that.
		try:
			MESSAGE = max(Artist,Name,Album, key=len)
		except:
			MESSAGE = "Starting up"
		# length exceeds WIDTH we need to enable scrolling.
		# All three lines will scroll
		size_x, size_y = draw.textsize(MESSAGE, font)
		x = ((time.time() - t_start) * 20) 
		x %= (size_x + disp.width)
		draw.rectangle((0,0,WIDTH,HEIGHT), outline=0, fill=0)
		
		scroll(Artist,0,x,MESSAGE)
		scroll(Name,13,x,MESSAGE)
		scroll(Album,26,x,MESSAGE)

		SampleRate = float(SampleRate) / 1000
		draw.text((0, text_y+53), str(SampleRate)  + "kHz, " + str(Bitdepth) + "bit, " + str(Bitrate) + "kbps" ,  font=fontB, fill=255)
		
		#Progress bar
		#During startup or Radio/Streamin not all fields are present/filled, so we need to deal with that.
		#For radio/streaming I'll place the bar at 75% (Except)
		try:
			end = 105 / float(DurationMS )
			pos = end * float(PositionMS)
		except:
			end = 105 / float(1000)
			pos = end * float(750)
		#Check to avoid 0
		if pos < 0:
			pos = 75
		elif pos < 1:
			pos = 1
		# to avoid a time consuming loop, we just play with the width of the bar picture
		placeimage(0,47,int(pos),5,"/home/pi/oled/bar.png")
	
		#Play/Pause/Wait button: Load correct Iamge
		if Status == 'Playing':
			placeimage(110,45,20,20,"/home/pi/oled/Play.png")
		if Status == 'Paused':
			placeimage(110,45,20,20,"/home/pi/oled/Pause.png")
		if Status == 'Waiting':
			placeimage(110,45,15,20,"/home/pi/oled/hg1.png")
	#Capture all other states of JRiver
	elif Status == 'Buffering...' or Status == 'Opening...' or Status == 'Aborting, Please wait':
		draw.rectangle((0,0,WIDTH,HEIGHT), outline=0, fill=0)
		draw.text((10, text_y+48), Status,  font=fontC, fill=255)
		placeimage(54,5,20,32,"/home/pi/oled/hg1.png")
		dispimage(0.5)
		placeimage(54,5,20,32,"/home/pi/oled/hg2.png")
		dispimage(0.5)
		placeimage(54,5,20,32,"/home/pi/oled/hg3.png")
		dispimage(0.5)
		placeimage(54,5,20,32,"/home/pi/oled/hg4.png")
		dispimage(0.5)
	#If there is nothing to show, load standard screen
	else:
		draw.rectangle((0,0,WIDTH,HEIGHT), outline=0, fill=0)
		placeimage(0,5,128,44,"/home/pi/oled/logo.png")
		now = datetime.now()
		sTime = now.strftime("%H:%M:%S")
		sDate = now.strftime("%d-%b-%Y")
		draw.text((78, text_y+48), sTime,  font=fontA, fill=255)
		draw.text((0, text_y+48), sDate,  font=fontA, fill=255)
		
	dispimage(0.5)